/*
Author: 	Fly 
Datum:      06.10.06
Programm:	rgb.c
Systemtakt:	8 MHz, intern




//xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Funktion:	Dimmt 10 Farbtne automatisch, siehe Video ****** Mein 1.C-Projektchen
//xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


Hardware:   Mega 16
PA0:        npn TR schaltet R-Led ber 100 Ohm
PA1:        npn TR schaltet G-Led ber 750 Ohm 
PA2:        npn TR schaltet B-Led ber 560 Ohm


H:          der Farbton als Farbwinkel H auf dem Farbkreis (z. B. 0 = Rot, 120 = Grn, 240 = Blau) 
S:          die Sttigung S in Prozent (z. B. 0% = keine Farbe, 50% = ungesttigte Farbe, 100% = gesttigte, reine Farbe) 
V:          der Grauwert V als Prozentwert angegeben (z. B. 0% = keine Helligkeit, 100% = volle Helligkeit)

Skalierung der HSV Werte:
H:			0-360,  float Auflsung
S:			0-1, 	float Auflsung, nicht verndern
V:			0-1, 	float Auflsung	

Weil ich C-Anfnger: (Gehirnsttze)
x |= (1 << Bitnummer);  // wird ein Bit in x gesetzt
x &= ~(1 << Bitnummer); // wird ein Bit in x geloescht


*/


#define     Ledport             PORTA		// RGB Led Port
#define     Inputport           PINC		// Noch nicht verwirklicht
#define     DDR_Ledport         DDRA
#define     DDR_Inputport       DDRC
#define     R                   0			// R Ausgang
#define     G                   1			// G
#define     B                   2			// B
#define     Get_New_RGB_Value   1			// Statusbit

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <math.h>



// Prototypen
void hsv_to_rgb (void);


// Globale Variabeln (Timer ISR)
volatile int		Red, Green,    Blue;             // PWM Register
volatile float      H,   S,        V;                        
volatile uint8_t    Statusreg;
volatile int        PWM_cnt, timebase;              // Counter Einschaltzeit

	

//-------------------------------------------------------------------------------
//Hauptprogramm
//-------------------------------------------------------------------------------
int main(void)
{
 DDR_Ledport=255;   Ledport=0;  						// Led aus
 PWM_cnt=0;    Statusreg=0;								// Variabel init.

 H=0, S=1, V=1;											// HSV Startwerte, rot, volle Helligkeit
 hsv_to_rgb();											// HSV to RGB

 TCCR0=0; TCNT0=255; TCCR0=4; TIMSK = _BV(TOIE0);       // T0 Starten, Overflow Interrupt f/256
 sei();                                                 // Enable Interrupts

 while(1)
 {
  if (Statusreg == Get_New_RGB_Value)                   // Wenn HSV Werte gendert
       { hsv_to_rgb(); Statusreg=0; }					// HSV to RGB, Status zurcksetzen
  }
}
//-------------------------------------------------------------------------------
//Hauptprogramm
//-------------------------------------------------------------------------------


//-------------------------------------------------------------------------------
//HSV nach RGB konvertieren
//-------------------------------------------------------------------------------


void hsv_to_rgb(void)	
{
 float h=H; 			// HSV Werte kopieren
 float s=S;
 float v=V;

 float r,g,b;			

 int i;
 float f, p, q, t;

 if( s == 0 ) 
 	{
	r = g = b = v;
	return;
	}

 h /= 60;				// sector 0 to 5
 i = floor( h );
 f = h - i;			
 p = v * ( 1 - s );
 q = v * ( 1 - s * f );
 t = v * ( 1 - s * ( 1 - f ) );

 switch( i )
 	 	{
		case 0:
			r = v; g = t; b = p; break;
		
		case 1:
			r = q; g = v; b = p; break;
			
		case 2:
			r = p; g = v; b = t; break;
				
		case 3:
			r = p; g = q; b = v; break;			
				
		case 4:
			r = t; g = p; b = v; break;
								
		default:								
			r = v; g = p; b = q; break;				// case 5:					
		}

 
 int rc=r*255;										// Auf 8 Bit RGB skalieren, nacher nur noch kopieren
 int gc=g*255;
 int bc=b*255;		
  
 cli();                                              // Kein Interrupt beim bernehmen der neuen Werte!
 Red=rc, Green=gc, Blue=bc;      
 sei();

}

//-------------------------------------------------------------------------------
//HSV nach RGB konvertieren
//-------------------------------------------------------------------------------




//-------------------------------------------------------------------------------
//Timer  0 Interrupt, f/256
//-------------------------------------------------------------------------------

SIGNAL (SIG_OVERFLOW0)
{
if ((PWM_cnt <=Red) && (Red!=0))
            { Ledport |= (1 << R);}                             // Rot aktiv
            else {Ledport &= ~(1 << R);}                        // Rot inaktiv

        if ((PWM_cnt <=Green) && (Green!=0))
            { Ledport |= (1 << G);}                       
            else {Ledport &= ~(1 << G);}     
        
        if ((PWM_cnt <=Blue) && (Blue!=0))
            { Ledport |= (1 << B);}                        
            else {Ledport &= ~(1 << B);}                  

        if (PWM_cnt==255)										// Wenn PWM Periode vorbei 
            {
              PWM_cnt=0; timebase++;							// Neustart
              if (timebase == 2)                                // Geschwindigkeit des Ablaufs
                  {
				   timebase=0; V=(V-0.005);  					// Helligkeit dimmen, Geschwindigkeit abhngig von timebase, Auflsung beliebig (float)
                   if (V<=0) 									// Wenn Helligkeit = 0
				   		{
						 V=1; H=(H+36);							// Volle Helligkeit, Neuer Farbton, 10 Farbtne, Auflsund beliebig vernderbar (float)
						 if (H>=360) {H=0;}
						 }								
                   Statusreg = Get_New_RGB_Value;				// Neue HSV Werte Stausbit aktiv
                  }                                           
            }

            else
            {PWM_cnt++;}
              
        TCCR0=0; TCNT0=255; TCCR0=4;		                  	// f:256, Timer0 starten
        TIMSK = _BV(TOIE0);                                     // Timer0 Overflow INT erlauben

} 
//-------------------------------------------------------------------------------
//Timer  0 Interrupt
//-------------------------------------------------------------------------------





